@@ -3,6 +3,7 @@ import logging
 import unittest
 from datetime import datetime
 from io import BytesIO
+from pathlib import Path
 
 import PIL
 from PIL.Image import Image
@@ -10,10 +11,10 @@ from bs4 import BeautifulSoup
 from bs4.element import Tag
 from requests import Response
 
-from spoofbot import Firefox, Chrome, MimeTypeTag, Windows, MacOSX, Linux
-from spoofbot.adapter import load_har, HarAdapter
+from spoofbot import Firefox, Chrome, MimeTypeTag, Windows, MacOSX, Linux, \
+    MacOSXVersion, WindowsVersion
+from spoofbot.adapter import HarCache
 from spoofbot.util import encode_form_data, TimelessRequestsCookieJar
-from tests.config import resolve_path
 
 logging.basicConfig(level=logging.DEBUG)
 # noinspection SpellCheckingInspection
@@ -23,41 +24,48 @@ logging.getLogger('chardet.charsetprober').setLevel(logging.INFO)
 # noinspection SpellCheckingInspection
 logging.getLogger('chardet.universaldetector').setLevel(logging.INFO)
 
+p = Path(__file__).parent.parent
+
 
 class WuxiaWorldTest(unittest.TestCase):
+    browser: Firefox = None
+
     @classmethod
     def setUpClass(cls) -> None:
         cls.browser = Firefox(ff_version=(71, 0))
-        cls.browser.cookies = TimelessRequestsCookieJar(datetime(2019, 12, 21, 12, 0, 0))
+        cls.browser.cookies = TimelessRequestsCookieJar(
+            datetime(2019, 12, 21, 12, 0, 0)
+        )
         cls.browser._accept = [
             MimeTypeTag("text", "html"),
             MimeTypeTag("application", "xhtml+xml"),
             MimeTypeTag("application", "xml", q=0.9),
             MimeTypeTag("*", "*", q=0.8)
         ]
-        cls.browser.adapter = HarAdapter(load_har(resolve_path('../test_data/www.wuxiaworld.com_Archive_ALL.har')))
+        cls.browser.adapter = HarCache(p /
+                                       'test_data/www.wuxiaworld.com_Archive_ALL.har')
 
     def test_01_main_site(self):
         self.browser.transfer_encoding = 'Trailers'
-        resp = self.browser.navigate('http://wuxiaworld.com/')
+        resp = self.browser.navigate('http://wuxiaworld.com/')[0]
         self.assertEqual(200, resp.status_code)
 
     def test_02_login(self):
-        resp = self.browser.navigate('https://www.wuxiaworld.com/account/login')
+        resp = self.browser.navigate('https://www.wuxiaworld.com/account/login')[0]
         self.assertEqual(200, resp.status_code)
 
-        with open(resolve_path('../test_data/ww_auth.json'), 'r') as fp:
+        with open(p / 'test_data/ww_auth.json', 'r') as fp:
             auth = json.load(fp)
         doc = BeautifulSoup(resp.text, features="html.parser")
         # noinspection PyTypeChecker
         rvt_input: Tag = doc.select_one('input[name="__RequestVerificationToken"]')
-        data = [
+        data = '&'.join(map('='.join, [
             ('Email', auth['Email']),
             ('Password', auth['Password']),
             ('RememberMe', 'true'),
             ('__RequestVerificationToken', rvt_input.get('value')),
             ('RememberMe', 'false')
-        ]
+        ]))
         self.browser.adapter.match_header_order = False
         self.browser.adapter.encode_post_data = True
         resp = self.browser.post('https://www.wuxiaworld.com/account/login', headers={
@@ -68,10 +76,10 @@ class WuxiaWorldTest(unittest.TestCase):
 
     def test_03_claim_karma(self):
         self.browser.adapter.match_header_order = True
-        resp = self.browser.navigate('https://www.wuxiaworld.com/profile/karma')
+        resp = self.browser.navigate('https://www.wuxiaworld.com/profile/karma')[0]
         self.assertEqual(200, resp.status_code)
 
-        resp = self.browser.navigate('https://www.wuxiaworld.com/profile/missions')
+        resp = self.browser.navigate('https://www.wuxiaworld.com/profile/missions')[0]
         self.assertEqual(200, resp.status_code)
 
         doc = BeautifulSoup(resp.text, features="html.parser")
@@ -82,9 +90,10 @@ class WuxiaWorldTest(unittest.TestCase):
             ('__RequestVerificationToken', rvt_input.get('value')),
         ]
         self.browser.adapter.match_header_order = False
-        resp = self.browser.post('https://www.wuxiaworld.com/profile/missions', headers={
-            'Content-Type': 'application/x-www-form-urlencoded'
-        }, data=encode_form_data(data))
+        resp = self.browser.post('https://www.wuxiaworld.com/profile/missions',
+                                 headers={
+                                     'Content-Type': 'application/x-www-form-urlencoded'
+                                 }, data=encode_form_data(data))
         self.assertEqual(200, resp.status_code)
 
     def test_04_logout(self):
@@ -97,37 +106,41 @@ class WuxiaWorldTest(unittest.TestCase):
 
     def test_05_novels(self):
         self.browser.upgrade_insecure_requests = True
-        resp = self.browser.navigate('https://www.wuxiaworld.com/novels')
+        resp = self.browser.navigate('https://www.wuxiaworld.com/novels')[0]
         self.assertEqual(200, resp.status_code)
 
     def test_06_search(self):
         self.browser.upgrade_insecure_requests = False
-        resp = self.browser.post('https://www.wuxiaworld.com/api/novels/search', headers={
-            'Accept': 'application/json, text/plain, */*',
-            'Content-Type': 'application/json;charset=utf-8',
-        }, data=json.dumps({
-            "title": "",
-            "tags": [],
-            "genres": [],
-            "active": None,
-            "sortType": "Name",
-            "sortAsc": True,
-            "searchAfter": None,
-            "count": 15
-        }, separators=(',', ':')))
+        resp = self.browser.post('https://www.wuxiaworld.com/api/novels/search',
+                                 headers={
+                                     'Accept': 'application/json, text/plain, */*',
+                                     'Content-Type': 'application/json;charset=utf-8',
+                                 }, data=json.dumps({
+                "title": "",
+                "tags": [],
+                "genres": [],
+                "active": None,
+                "sortType": "Name",
+                "sortAsc": True,
+                "searchAfter": None,
+                "count": 15
+            }, separators=(',', ':')))
         self.assertEqual(200, resp.status_code)
 
     # noinspection SpellCheckingInspection
     def test_07_btth(self):
         self.browser.upgrade_insecure_requests = True
-        resp = self.browser.navigate('https://www.wuxiaworld.com/novel/battle-through-the-heavens')
+        resp = self.browser.navigate(
+            'https://www.wuxiaworld.com/novel/battle-through-the-heavens'
+        )[0]
         self.assertEqual(200, resp.status_code)
 
     # noinspection SpellCheckingInspection
     def test_08_btth_cover(self):
         self.browser.upgrade_insecure_requests = False
         resp = self.browser.get(
-            'https://cdn.wuxiaworld.com/images/covers/btth.jpg?ver=b49ecfeb59e7f8a1e94379b5bfa58828e70883a4',
+            'https://cdn.wuxiaworld.com/images/covers/btth.jpg'
+            '?ver=b49ecfeb59e7f8a1e94379b5bfa58828e70883a4',
             headers={
                 'Accept': 'image/webp,*/*',
                 'TE': None,
@@ -140,18 +153,28 @@ class WuxiaWorldTest(unittest.TestCase):
 
     def test_09_chapters_1_and_2(self):
         self.browser.upgrade_insecure_requests = True
-        resp = self.browser.navigate('https://www.wuxiaworld.com/novel/battle-through-the-heavens/btth-chapter-1')
+        resp = self.browser.navigate(
+            'https://www.wuxiaworld.com/novel/'
+            'battle-through-the-heavens/btth-chapter-1'
+        )[0]
         self.assertEqual(200, resp.status_code)
 
-        resp = self.browser.navigate('https://www.wuxiaworld.com/novel/battle-through-the-heavens/btth-chapter-2')
+        resp = self.browser.navigate(
+            'https://www.wuxiaworld.com/novel/'
+            'battle-through-the-heavens/btth-chapter-2'
+        )[0]
         self.assertEqual(200, resp.status_code)
 
 
 class ChromeTest(unittest.TestCase):
+    browser: Chrome = None
+    duckduckgo_navigate: Response = None
+    httpbin_navigate: Response = None
+
     @classmethod
     def setUpClass(cls) -> None:
-        cls.browser = Chrome()
-        cls.browser.adapter = HarAdapter(load_har(resolve_path('test_data/chrome_full.har')))
+        cls.browser = Chrome(chrome_version=(79, 0, 3945, 130))
+        cls.browser.adapter = HarCache(p / 'test_data/chrome_full.har')
         cls.duckduckgo_navigate = Response()
         cls.duckduckgo_navigate.url = 'https://duckduckgo.com/'
         cls.httpbin_navigate = Response()
@@ -201,11 +224,12 @@ class ChromeTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = True
         self.browser.connection = ''
         self.browser._last_navigate = self.duckduckgo_navigate
-        self.assertIsNotNone(self.browser.get('https://duckduckgo.com/?q=httpbin&t=h_', headers={
-            'Host': None,
-            'Sec-Fetch-User': '?1',
-            'Sec-Fetch-Mode': 'navigate',
-        }))
+        self.assertIsNotNone(
+            self.browser.get('https://duckduckgo.com/?q=httpbin&t=h_', headers={
+                'Host': None,
+                'Sec-Fetch-User': '?1',
+                'Sec-Fetch-Mode': 'navigate',
+            }))
 
     def test_04_httpbin(self):
         self.browser.transfer_encoding = None
@@ -245,7 +269,8 @@ class ChromeTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.patch('https://httpbin.org/patch', headers={'Accept': 'application/json'}))
+            self.browser.patch('https://httpbin.org/patch',
+                               headers={'Accept': 'application/json'}))
 
     def test_08_httpbin_post(self):  # Had to add Content-Length header to HAR
         self.browser.adapter.match_header_order = False
@@ -253,7 +278,8 @@ class ChromeTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.post('https://httpbin.org/post', headers={'Accept': 'application/json'}))
+            self.browser.post('https://httpbin.org/post',
+                              headers={'Accept': 'application/json'}))
 
     def test_09_httpbin_put(self):  # Had to add Content-Length header to HAR
         self.browser.adapter.match_header_order = False
@@ -261,7 +287,8 @@ class ChromeTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.put('https://httpbin.org/put', headers={'Accept': 'application/json'}))
+            self.browser.put('https://httpbin.org/put',
+                             headers={'Accept': 'application/json'}))
 
     def test_10_httpbin_basic_auth(self):
         self.browser.adapter.match_header_order = False
@@ -269,11 +296,12 @@ class ChromeTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         # noinspection SpellCheckingInspection
-        self.assertIsNotNone(self.browser.get('https://httpbin.org/basic-auth/admin/password', headers={
-            'Accept': 'application/json',
-            'Sec-Fetch-Mode': 'cors',
-            # 'Authorization': 'Basic YWRtaW46cGFzc3dvcmQ=',
-        }))
+        self.assertIsNotNone(
+            self.browser.get('https://httpbin.org/basic-auth/admin/password', headers={
+                'Accept': 'application/json',
+                'Sec-Fetch-Mode': 'cors',
+                # 'Authorization': 'Basic YWRtaW46cGFzc3dvcmQ=',
+            }))
 
     def test_11_httpbin_status_200(self):
         self.browser.adapter.match_header_order = False
@@ -512,6 +540,9 @@ class ChromeTest(unittest.TestCase):
         })
         self.assertIsNotNone(response)
 
+    # https://ipinfo.io/
+    # aHR0cHM6Ly9pcGluZm8uaW8v
+    # https%3a%2f%2fipinfo.io%2f
     def test_32_forms(self):
         self.browser.adapter.match_header_order = False
         self.browser.adapter.match_headers = True
@@ -541,10 +572,14 @@ class ChromeTest(unittest.TestCase):
 
 
 class FirefoxTest(unittest.TestCase):
+    browser: Firefox = None
+    duckduckgo_navigate: Response = None
+    httpbin_navigate: Response = None
+
     @classmethod
     def setUpClass(cls) -> None:
-        cls.browser = Firefox()
-        cls.browser.adapter = HarAdapter(load_har(resolve_path('../test_data/ff_full.har')))
+        cls.browser = Firefox(ff_version=(72, 0))
+        cls.browser.adapter = HarCache(p / 'test_data/ff_full.har')
         cls.duckduckgo_navigate = Response()
         cls.duckduckgo_navigate.url = 'https://duckduckgo.com/'
         cls.httpbin_navigate = Response()
@@ -596,7 +631,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.transfer_encoding = None
         self.browser.upgrade_insecure_requests = True
         self.browser._last_navigate = self.duckduckgo_navigate
-        self.assertIsNotNone(self.browser.navigate('https://httpbin.org/', headers={'Origin': None}))
+        self.assertIsNotNone(
+            self.browser.navigate('https://httpbin.org/', headers={'Origin': None}))
 
     def test_05_httpbin_delete(self):  # Had to add Content-Length header to HAR
         self.browser.adapter.match_header_order = False
@@ -604,7 +640,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.delete('https://httpbin.org/delete', headers={'Accept': 'application/json'}))
+            self.browser.delete('https://httpbin.org/delete',
+                                headers={'Accept': 'application/json'}))
 
     def test_06_httpbin_get(self):
         self.browser.adapter.match_header_order = False
@@ -612,7 +649,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.get('https://httpbin.org/get', headers={'Accept': 'application/json'}))
+            self.browser.get('https://httpbin.org/get',
+                             headers={'Accept': 'application/json'}))
 
     def test_07_httpbin_patch(self):  # Had to add Content-Length header to HAR
         self.browser.adapter.match_header_order = False
@@ -620,7 +658,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.patch('https://httpbin.org/patch', headers={'Accept': 'application/json'}))
+            self.browser.patch('https://httpbin.org/patch',
+                               headers={'Accept': 'application/json'}))
 
     def test_08_httpbin_post(self):  # Had to add Content-Length header to HAR
         self.browser.adapter.match_header_order = False
@@ -628,7 +667,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.post('https://httpbin.org/post', headers={'Accept': 'application/json'}))
+            self.browser.post('https://httpbin.org/post',
+                              headers={'Accept': 'application/json'}))
 
     def test_09_httpbin_put(self):  # Had to add Content-Length header to HAR
         self.browser.adapter.match_header_order = False
@@ -636,7 +676,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         self.assertIsNotNone(
-            self.browser.put('https://httpbin.org/put', headers={'Accept': 'application/json'}))
+            self.browser.put('https://httpbin.org/put',
+                             headers={'Accept': 'application/json'}))
 
     def test_10_httpbin_basic_auth(self):
         self.browser.adapter.match_header_order = False
@@ -644,17 +685,19 @@ class FirefoxTest(unittest.TestCase):
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
         # noinspection SpellCheckingInspection
-        self.assertIsNotNone(self.browser.get('https://httpbin.org/basic-auth/admin/password', headers={
-            'Accept': 'application/json',
-            'Authorization': 'Basic YWRtaW46cGFzc3dvcmQ=',
-        }))
+        self.assertIsNotNone(
+            self.browser.get('https://httpbin.org/basic-auth/admin/password', headers={
+                'Accept': 'application/json',
+                'Authorization': 'Basic YWRtaW46cGFzc3dvcmQ=',
+            }))
 
     def test_11_httpbin_status_200(self):
         self.browser.adapter.match_header_order = False
         self.browser.transfer_encoding = None
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
-        response = self.browser.get('https://httpbin.org/status/200', headers={'Accept': 'text/plain'})
+        response = self.browser.get('https://httpbin.org/status/200',
+                                    headers={'Accept': 'text/plain'})
         self.assertIsNotNone(response)
         self.assertEqual(200, response.status_code)
 
@@ -663,7 +706,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.transfer_encoding = None
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
-        response = self.browser.get('https://httpbin.org/status/300', headers={'Accept': 'text/plain'})
+        response = self.browser.get('https://httpbin.org/status/300',
+                                    headers={'Accept': 'text/plain'})
         self.assertIsNotNone(response)
         self.assertEqual(300, response.status_code)
 
@@ -672,7 +716,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.transfer_encoding = None
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
-        response = self.browser.get('https://httpbin.org/status/400', headers={'Accept': 'text/plain'})
+        response = self.browser.get('https://httpbin.org/status/400',
+                                    headers={'Accept': 'text/plain'})
         self.assertIsNotNone(response)
         self.assertEqual(400, response.status_code)
 
@@ -681,7 +726,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.transfer_encoding = None
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
-        response = self.browser.get('https://httpbin.org/status/500', headers={'Accept': 'text/plain'})
+        response = self.browser.get('https://httpbin.org/status/500',
+                                    headers={'Accept': 'text/plain'})
         self.assertIsNotNone(response)
         self.assertEqual(500, response.status_code)
 
@@ -690,7 +736,8 @@ class FirefoxTest(unittest.TestCase):
         self.browser.transfer_encoding = None
         self.browser.upgrade_insecure_requests = False
         self.browser._last_navigate = self.httpbin_navigate
-        response = self.browser.get('https://httpbin.org/brotli', headers={'Accept': 'application/json'})
+        response = self.browser.get('https://httpbin.org/brotli',
+                                    headers={'Accept': 'application/json'})
         self.assertIsNotNone(response)
         self.assertTrue(response.json()['brotli'])
 
@@ -699,25 +746,25 @@ class FirefoxUserAgentTest(unittest.TestCase):
     def test_default(self):
         self.assertEqual(
             'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:71.0) Gecko/20100101 Firefox/71.0',
-            Firefox.create_user_agent()
+            Firefox.create_user_agent(version=(71, 0))
         )
 
     def test_win(self):
         self.assertEqual(
             'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:71.0) Gecko/20100101 Firefox/71.0',
-            Firefox.create_user_agent(Windows())
+            Firefox.create_user_agent(Windows(WindowsVersion.Win11), version=(71, 0))
         )
 
     def test_mac(self):
         self.assertEqual(
-            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15; rv:71.0) Gecko/20100101 Firefox/71.0',
-            Firefox.create_user_agent(MacOSX())
+            'Mozilla/5.0 (Macintosh; Intel Mac OS X 12; rv:71.0) Gecko/20100101 Firefox/71.0',
+            Firefox.create_user_agent(MacOSX(MacOSXVersion.Monterey), version=(71, 0))
         )
 
     def test_linux(self):
         self.assertEqual(
             'Mozilla/5.0 (X11; Linux x86_64; rv:71.0) Gecko/20100101 Firefox/71.0',
-            Firefox.create_user_agent(Linux())
+            Firefox.create_user_agent(Linux(), version=(71, 0))
         )
 
     def test_69_42(self):
@@ -729,7 +776,7 @@ class FirefoxUserAgentTest(unittest.TestCase):
     def test_20181001000000(self):
         self.assertEqual(
             'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:71.0) Gecko/20181001000000 Firefox/71.0',
-            Firefox.create_user_agent(build_id=20181001000000)
+            Firefox.create_user_agent(version=(71, 0), build_id=20181001000000)
         )
 
 
@@ -737,25 +784,28 @@ class ChromeUserAgentTest(unittest.TestCase):
     def test_default(self):
         self.assertEqual(
             'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/537.36',
-            Chrome.create_user_agent()
+            Chrome.create_user_agent(version=(79, 0, 3945, 130))
         )
 
     def test_win(self):
         self.assertEqual(
             'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/537.36',
-            Chrome.create_user_agent(Windows())
+            Chrome.create_user_agent(Windows(WindowsVersion.Win11),
+                                     version=(79, 0, 3945, 130))
         )
 
     def test_mac(self):
         self.assertEqual(
-            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/537.36',
-            Chrome.create_user_agent(MacOSX())
+            'Mozilla/5.0 (Macintosh; Intel Mac OS X 12) AppleWebKit/537.36 (KHTML, '
+            'like Gecko) Chrome/79.0.3945.130 Safari/537.36',
+            Chrome.create_user_agent(MacOSX(MacOSXVersion.Monterey),
+                                     version=(79, 0, 3945, 130))
         )
 
     def test_linux(self):
         self.assertEqual(
             'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/537.36',
-            Chrome.create_user_agent(Linux())
+            Chrome.create_user_agent(Linux(), version=(79, 0, 3945, 130))
         )
 
     def test_69_42(self):
@@ -767,7 +817,8 @@ class ChromeUserAgentTest(unittest.TestCase):
     def test_20181001000000(self):
         self.assertEqual(
             'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/536.5',
-            Chrome.create_user_agent(webkit_version=(536, 5))
+            Chrome.create_user_agent(version=(79, 0, 3945, 130),
+                                     webkit_version=(536, 5))
         )
 
 
